local old_CloseSpecialWindows;
local NUMDKPROWS = 17;
local ScrollPaneBackdrop  = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 5, bottom = 3 }
};
		
function sdkp:ShowBids()
	local f = sdkp.newui;
	if not f then 
		-- build window
		local AceGUI = LibStub("AceGUI-3.0");
		f = AceGUI:Create("Frame");
		local window = f.frame;
		f:SetTitle("SDKP");
		-- I don't like the ace3 defaults
		window:SetWidth(575);
		window:SetHeight(400);
		f.sizer_se:Hide();
		f.sizer_e:Hide();
		f.sizer_s:Hide();
		f.statusbg:EnableMouse();
		f.statusbg:SetScript("OnMouseDown",f.title:GetScript("OnMouseDown"));
		f.statusbg:SetScript("OnMouseUp",f.title:GetScript("OnMouseUp"));
		
		sdkp:BuildSlideFrame(window);
		
		local columns = {
			{ ["name"] = "Name", ["width"] = 115, ["defaultsort"] = "dsc", }, -- [1]
			{ ["name"] = "Main", ["width"] = 115, ["defaultsort"] = "dsc", }, -- [2]
			{ ["name"] = "TDA", ["width"] = 55,	["align"] = "RIGHT", ["sortnext"] = 4, }, -- [3]
			{ ["name"] = "SDA", ["width"] = 55, ["align"] = "RIGHT", ["sortnext"] = 5,}, -- [4]
			{ ["name"] = "NDA", ["width"] = 55, ["align"] = "RIGHT", ["sortnext"] = 7,}, -- [5]
			{ 
				["name"] = "Bid", 
				["width"] = 55, 
				["align"] = "RIGHT",
				["bgcolor"] = { ["r"] = 0.0, ["g"] = 0.0, ["b"] = 0.0, ["a"] = 0.5 },
				["sortnext"] = 3,
			}, -- [6]
			{ ["name"] = "DKP", ["width"] = 55, ["align"] = "RIGHT", ["sortnext"] = 1,}, -- [7]
		};
		self.BIDSCROLLFRAME = ScrollingTable:CreateST(columns, 12, 16, nil, window);
		self.BIDSCROLLFRAME.frame:SetPoint("TOP", window, "CENTER", 0, 46);
		self.BIDSCROLLFRAME.frame:SetPoint("LEFT", window, "LEFT", 15, 0);
		
		sdkp:UpdateScrollData();
		if self.db.profile.currentfilter and SDKP_FILTERS[self.db.profile.currentfilter] then 
			self.BIDSCROLLFRAME:SetFilter(SDKP_FILTERS[self.db.profile.currentfilter]);
		end
		if sdkp.db.profile.currentsystem then 
			self.BIDSCROLLFRAME:SetData(SDKPDATA[sdkp.db.profile.currentsystem]);
		end
		
		-- create the system dropdown
		local system = AceGUI:Create("Dropdown");
		system.frame:SetWidth(170);
		local list = {};
		for sys in pairs(SDKP_SYSTEMS) do
			tinsert(list, sys);
		end
		system:SetList(list);
		system:SetText("");
		if self.db and self.db.profile.currentsystem then 
			system:SetText(self.db.profile.currentsystem);
		end
		system:SetLabel("Choose a system:");
		system:SetCallback("OnValueChanged",
			function(context, event, value)
				self.db.profile.currentsystem = context.list[value];
				sdkp:UpdateScrollData(); -- we should be doing this when the raid changes, but what the hell...
				self.BIDSCROLLFRAME:SetData(SDKPDATA[sdkp.db.profile.currentsystem]);
			end
		);
		f:AddChild(system);
		
		-- create the filter dropdown
		local filter = AceGUI:Create("Dropdown");
		filter.frame:SetWidth(170);
		filter:SetList(SDKP_FILTERS_LIST);
		filter:SetText("");
		if self.db and self.db.profile.currentfilter then 
			filter:SetText(self.db.profile.currentfilter);
		end
		filter:SetLabel("Choose a display filter:");
		filter:SetCallback("OnValueChanged",
			function(context, event, value)
				self.db.profile.currentfilter = context.list[value];
				sdkp:UpdateScrollData(); -- we should be doing this when the raid changes, but what the hell...
				self.BIDSCROLLFRAME:SetFilter(SDKP_FILTERS[context.list[value]]);
			end
		);
		f:AddChild(filter);
		
		-- and now, since acegui3 has retardedly few layout options(at time of writing), I must rape the framework.
		-- create edit box for bidding item
		local itemedit = CreateFrame("EditBox", "sdkpeditbox", window, "sdkpeditboxtemplate");
		itemedit:SetPoint("TOP", window, "TOP", 1, -42);
		
		local itemedropdown = CreateFrame("Button", "sdkplootdropdown", window, "sdkplootdropdowntemplate");
		itemedropdown:SetPoint("TOPLEFT", itemedit, "BOTTOMLEFT", 0, 10);
		itemedropdown:SetScript("OnShow", function() 
			sdkp:LootDropDown_OnShow(itemedropdown);
		end);
		itemedropdown:SetScript("OnClick", function() 
			sdkp:LootDropDown_OnClick(itemedropdown);
		end);
		itemedropdown:Hide(); itemedropdown:Show();
		
		-- create the start bids button
		local startbids = AceGUI:Create("Button");
		startbids.frame:SetWidth(130);
		startbids:SetText("Start Bids");
		startbids:SetCallback("OnClick",
			function(context, event)
				local item = itemedit:GetText();
				if context.text:GetText() == "Start Bids" then 
					self.BIDDING = true;
					context:SetText("Stop Bids");
					sdkp:SendRaidWarning("Bidding started for " .. item .. " ( full = " .. self:GetDkpForItem(item) .. " )");
					sdkp:SendRaidWarning("To bid: '/w " .. UnitName("player") .. " !bid X', where X is your bid.");
				else
					self.BIDDING = false;
					context:SetText("Start Bids");
					sdkp:SendRaidWarning("Bidding has ended for " .. item .. ".");
				end
			end
		);
		startbids.frame:SetParent(window);
		startbids.frame:SetPoint("TOPRIGHT", window, "TOPRIGHT", -20, -45);
		
		-- create the clear bids button
		local clearbids = AceGUI:Create("Button");
		clearbids.frame:SetWidth(130);
		clearbids:SetText("Clear Bids");
		clearbids:SetCallback("OnClick",
			function(context, event)
				itemedit:SetText("");
				self.db.profile.snapshot.bids = {};
				self.BIDSCROLLFRAME:SortData();
			end
		);
		clearbids.frame:SetParent(window);
		clearbids.frame:SetPoint("TOPRIGHT", window, "TOPRIGHT", -20, -90);
		
		sdkp.newui = f;
	end
	
	-- hide the fucking window on escape.   Why this isn't part of the Frame widget boggles my mind.
	if not old_CloseSpecialWindows then
		old_CloseSpecialWindows = CloseSpecialWindows
		CloseSpecialWindows = function()
			if f.frame:IsVisible() then 
				f:Hide(); return true;
			else
				return old_CloseSpecialWindows();
			end
		end
	end
	f:Show();
end

-- Reconstruct the data for display.   Use if the data in table is out of date.
function sdkp:UpdateScrollData()
	if SDKP_NOUPDATE_ZONE[GetRealZoneText()] or IsActiveBattlefieldArena() then 
		self:Debug("We don't track here!", GetRealZoneText());
		return;
	end
	
	local system = sdkp.db.profile.currentsystem;
	if system then
		local i = 1;
		local nRaidMembers = GetNumRaidMembers();
		for raidIndex = 1, nRaidMembers do
			local raidMember, _, _, _, class = GetRaidRosterInfo(raidIndex);
			local raidMemberLower = raidMember:lower();
			local sdkp_player_index = SDKPPLAYER[raidMemberLower];
			
			-- Add Raid members to the dkp list, if not there.
			if not sdkp_player_index then 
				self:Debug("UpdateScrollData:", raidMember, "not found, adding a stub.");
				local player_stub = { 
					["cols"] = { 
						{ ["value"] = raidMember, ["color"] = RAID_CLASS_COLORS[class:upper()]},
						{ ["value"] = "", },
						{ ["value"] = 0,},
						{ ["value"] = 0,},
						{ ["value"] = 0,},
						{ ["value"] = "", },
						{ ["value"] = 0,},
					},
					["color"] = sdkp.COLOR_LOWPRIORITY,
				};
				
				-- Update with main's data
				local main = self:GetAltInfo(raidMember);
				if main and main ~= raidMember then 
					-- substitute in the main's data instead of the alt's data.
					local sdkp_main_index = SDKPPLAYER[main:lower()];
					if sdkp_main_index then
						self:Debug("UpdateScrollData: Main for", raidMember, "=> ", main);
						local mdata = SDKPDATA[system][sdkp_main_index];
						player_stub.cols[2] = mdata.cols[1];
						for col = 3, #mdata.cols do
							player_stub.cols[col] = mdata.cols[col];
						end
					else
						self:Debug("UpdateScrollData: Main for", raidMember, "not found :(");
					end
				end 
				for system, data in pairs(SDKPDATA) do 
					table.insert(data, player_stub);
					SDKPPLAYER[raidMemberLower] = #data;
				end
			end
		end
		self.BIDSCROLLFRAME:SortData();
	end
end

function sdkp:LootDropDown_OnShow(frame)
	UIDropDownMenu_Initialize(frame, function()
		if not sdkp.LOOT_WINDOW then 
			sdkp.LOOT_WINDOW = {}; 
		end
		
		for link in pairs(sdkp.LOOT_WINDOW) do
			UIDropDownMenu_AddButton({
				text = link,
				value = link,
				func = sdkp_LootDropDownSelect,
				notCheckable = nil,
				keepShownOnClick = nil,
			});
		end
	end);
end

function sdkp:LootDropDown_OnClick(frame)
	ToggleDropDownMenu(1, nil, frame, "sdkpeditbox", 0, 0);
end

function sdkp_LootDropDownSelect()
	getglobal("sdkpeditbox"):SetText(this.value);
end

function sdkp:BuildSlideFrame(parent)
	local SlidePaneBackdrop = {
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
		edgeFile = "Interface\\AddOns\\sdkp\\textures\\sdkp-slideout-border",
		tile = true, tileSize = 32, edgeSize = 32,
		insets = { left = 3, right = 3, top = 5, bottom = 3 }
	};
	local SlidePaneBackdropSlim = {
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
		edgeFile = "Interface\\AddOns\\sdkp\\textures\\sdkp-slideout-border-slim",
		tile = true, tileSize = 32, edgeSize = 32,
		insets = { left = 3, right = 3, top = 5, bottom = 3 }
	};
	
	local OPEN, CLOSED = 190, 20;
	-- slide frame
	local slideframe =  CreateFrame("Frame", "slideframe", parent);
	slideframe:EnableMouse();
	slideframe:SetFrameStrata("FULLSCREEN");
	slideframe:SetWidth(CLOSED);
	slideframe:SetHeight(350);
	slideframe:SetBackdrop(SlidePaneBackdropSlim);
	slideframe:SetPoint("LEFT", parent, "RIGHT", -8, 0);
	
	-- build slider scroll frame
	local scrollframebg = CreateFrame("Frame", "slidescrollframebg", slideframe);
	scrollframebg:SetPoint("TOPLEFT", slideframe, "TOPLEFT", 10, -50);
	scrollframebg:SetPoint("BOTTOMRIGHT", slideframe, "BOTTOMRIGHT", -35, 15);
	scrollframebg:SetBackdrop(ScrollPaneBackdrop);
	scrollframebg:SetBackdropColor(0.1,0.1,0.1);
	scrollframebg:Hide();
	local scrollframe = CreateFrame("ScrollFrame", "slidescrollframe", scrollframebg, "FauxScrollFrameTemplate");
	scrollframe:SetPoint("TOPLEFT", scrollframebg, "TOPLEFT", 0, 0);
	scrollframe:SetPoint("BOTTOMRIGHT", scrollframebg, "BOTTOMRIGHT", -3, 0);
	scrollframe:SetScript("OnVerticalScroll", 
		function (self, offset)
			FauxScrollFrame_OnVerticalScroll(self, offset, 16, sdkp_RefreshSlideScrollFrame);
		end
	);

	
	local i;
	for i = 1, NUMDKPROWS do
		local rown = CreateFrame("Button", "sdkpscreenshotdisplay"..i, slideframe, "sdkpscreenshottemplate");
		local removefromall = getglobal("sdkpscreenshotdisplay"..i.."RemoveFromAll");
		local playerframe = getglobal("sdkpscreenshotdisplay"..i.."Name");
		removefromall:SetScript("OnClick", 
			function ()
				local player = playerframe:GetText();
				if sdkp.db.profile.snapshot.attendanceList[player] then 
					sdkp:CmdHandleManualSnapshot(player, "remove", sdkp.db.profile.snapshot.attendanceList);
				end
				sdkp:CmdHandleManualSnapshot(player, "remove", sdkp.db.profile.snapshot.dkpList);
			end
		);
		local removefromatt = getglobal("sdkpscreenshotdisplay"..i.."RemoveFromAtt");
		removefromatt:SetScript("OnClick", 
			function ()
				local player = playerframe:GetText();
				sdkp:CmdHandleManualSnapshot(player, "remove", sdkp.db.profile.snapshot.attendanceList);
				sdkp_RefreshSlideScrollFrame();
			end
		);
		local addtoatt = getglobal("sdkpscreenshotdisplay"..i.."AddToAtt");
		addtoatt:SetScript("OnClick", 
			function ()
				local player = playerframe:GetText();
				sdkp:CmdHandleManualSnapshot(player, "add", sdkp.db.profile.snapshot.attendanceList);
				sdkp_RefreshSlideScrollFrame();
			end
		);
		local relativeto = scrollframe;
		if i ~= 1 then 
			relativeto = getglobal("sdkpscreenshotdisplay"..(i-1));
			rown:SetPoint("TOPLEFT", relativeto, "TOPLEFT", 0, -16);
			rown:SetPoint("TOPRIGHT", relativeto, "TOPRIGHT", 0, -16);
		else
			rown:SetPoint("TOPLEFT", relativeto, "TOPLEFT", 0, -4);
			rown:SetPoint("TOPRIGHT", relativeto, "TOPRIGHT", -10, -4);
		end
		rown:Hide();
	end	
	
	-- close button
	local closebutton = CreateFrame("Button", "closebutton", slideframe);
	closebutton:SetNormalTexture("Interface\\AddOns\\sdkp\\textures\\sdkp-slider-button-close");
	closebutton:SetFrameStrata("FULLSCREEN_DIALOG");
	closebutton:SetPoint("CENTER", slideframe, "RIGHT", -6, 0);
	closebutton:SetWidth(64);
	closebutton:SetHeight(128);
	closebutton:Hide();
	
	-- open button
	local openbutton = CreateFrame("Button", "openbutton", slideframe);
	openbutton:SetNormalTexture("Interface\\AddOns\\sdkp\\textures\\sdkp-slider-button-open");
	openbutton:SetFrameStrata("FULLSCREEN_DIALOG");
	openbutton:SetPoint("CENTER", slideframe, "RIGHT", -6, 0);
	openbutton:SetWidth(64);
	openbutton:SetHeight(128);
	
	local slide = function()
		local step = 0.2 * OPEN;
		local startwidth = slideframe:GetWidth();
		local isTextureSwapped = false;
		local direction = 1;
		if startwidth >= OPEN then 
			direction = -1;
		end
		
		scrollframebg:Hide();
		for i = 1, NUMDKPROWS do getglobal("sdkpscreenshotdisplay"..i):Hide(); end
		slideframe:SetScript("OnUpdate", function(time)
			local width = slideframe:GetWidth() + (step * direction);
			if width >= OPEN then 
				width = OPEN;
			elseif width <= CLOSED then 
				width = CLOSED;
			end
			slideframe:SetWidth(width);
			
			if not isTextureSwapped then -- normal border texture overlaps wierd with small frame sizes.
				if width < 40 and direction < 0 then 
					slideframe:SetBackdrop(SlidePaneBackdropSlim);
					isTextureSwapped = true;
				elseif width > 40 and direction > 0 then 
					slideframe:SetBackdrop(SlidePaneBackdrop);
					isTextureSwapped = true;
				end
			end
			
			-- we're done moving, finish up.
			local buttons = {["-1"] = openbutton, ["1"] = closebutton};
			if width == CLOSED or width == OPEN then 
				buttons[""..direction]:Show();
				buttons[""..(direction*-1)]:Hide();
				slideframe:SetScript("OnUpdate", nil);
				if width == OPEN then 
					scrollframebg:Show();
					for i = 1, NUMDKPROWS do getglobal("sdkpscreenshotdisplay"..i):Show(); end
					sdkp:UpdateSlideScrollData(); 
				end
			end
		end);
	end;
	
	closebutton:SetScript("OnMouseUp", slide);
	openbutton:SetScript("OnMouseUp", slide);
end

-- display snapshot in the slidescrollframe
function sdkp_RefreshSlideScrollFrame()
	local scrollframe = getglobal("slidescrollframe");
	if scrollframe == nil then 
		return
	end
	
	local max = #sdkp.DkpDisplaySnapshot;
	FauxScrollFrame_Update(scrollframe, max, NUMDKPROWS, 16);
	local i, o = 0, FauxScrollFrame_GetOffset(scrollframe);
	
	for i = 1, NUMDKPROWS do
		local ref = i + o;
		local rdata = sdkp.DkpDisplaySnapshot[ref];
		local row = getglobal("sdkpscreenshotdisplay"..i);
		local column = getglobal("sdkpscreenshotdisplay"..i.."Name");
		if rdata then 
			local color = RAID_CLASS_COLORS[rdata.class:upper()];
			if not color then color = { r = 0.5, g = 0.5, b = 0.5}; end
			column:SetTextColor(color.r, color.g, color.b, 1.0);
			if scrollframe:IsVisible() then -- don't show the row if the panel is closed... duh
				 row:Show(); 
			end
			column:SetText(rdata.name);
			local removefromatt = getglobal("sdkpscreenshotdisplay"..i.."RemoveFromAtt");
			local addtoatt = getglobal("sdkpscreenshotdisplay"..i.."AddToAtt");
			if sdkp.db.profile.snapshot.attendanceList[rdata.name] then 
				removefromatt:Show();
				addtoatt:Hide();
			else
				removefromatt:Hide();
				addtoatt:Show();
			end
		else
			row:Hide();
			column:SetText("");
		end
	end
end

-- Reconstruct the data for display.   Use if the data in table is out of date.
function sdkp:UpdateSlideScrollData()
	local snapshot = sdkp.db.profile.snapshot;
	if snapshot then
		sdkp.DkpDisplaySnapshot = {};
		local i = 1;
		local raidIndex, nRaidMembers, RaidTable = 1, GetNumRaidMembers(), {};
		for player in pairs(snapshot.dkpList) do
			pdata = {
				["name"] = player,
				["class"] = "HUNTER", -- till i fix the att frame to use lib-st
			};			
			sdkp.DkpDisplaySnapshot[i] = pdata;	
			i = i + 1;
		end
		table.sort(sdkp.DkpDisplaySnapshot, function(a,b)
			return a.name < b.name;
		end);
	end
	if sdkp.newui then 
		sdkp_RefreshSlideScrollFrame(); 
	end
end
